from importlib import import_module
import logging
import simplejson as json
from case_conversion import pascalcase

from charpe.errors import InsuficientInformation, MisconfiguredProvider, \
    InsuficientInformation, MediumError


LOGGER = logging.getLogger(__name__)


class Handler:

    # this function runs in a different process...
    def __init__(self, config):
        self.config = config.copy()
        self.handlers = dict()

    # ...than this one, so no conexion can be shared between the two
    def __call__(self, channel, method, properties, body: bytes):
        medium = method.routing_key

        try:
            data = json.loads(body)
        except json.decoder.JSONDecodeError:
            return LOGGER.error('Couldn\'t decode event\'s JSON data')

        try:
            self.get_medium(medium).publish(data)
        except MisconfiguredProvider as e:
            return LOGGER.error(str(e))
        except InsuficientInformation as e:
            return LOGGER.error(str(e))
        except MediumError as e:
            return LOGGER.error(str(e))

    def get_medium(self, name):
        if name not in self.handlers:
            module_name = 'charpe.mediums.{}_handler'.format(name)
            class_name = pascalcase(name) + 'Handler'

            try:
                module = import_module(module_name)
                self.handlers[name] = getattr(module, class_name)(self.config)
            except ImportError:
                raise MisconfiguredProvider(
                    'Could not import provider module {}'.format(module_name)
                )
            except AttributeError:
                raise MisconfiguredProvider(
                    'Provider module {} does not define class {}'.format(
                        module_name,
                        class_name,
                    )
                )

        return self.handlers[name]



from multiprocessing import Pool
import signal
import pika
import logging

from charpe.handler import Handler

LOGGER = logging.getLogger(__name__)


class Loop:

    def __init__(self, config):
        self.config = config
        self.handler = Handler(config)

        LOGGER.info('Initialized loop')

    def start(self):
        connection = pika.BlockingConnection(pika.ConnectionParameters(
            host=self.config['RABBIT_HOST'],
        ))
        channel = connection.channel()

        channel.exchange_declare(
            exchange=self.config['RABBIT_NOTIFY_EXCHANGE'],
            exchange_type='direct'
        )
        LOGGER.info('Declared exchange {}'.format(
            self.config['RABBIT_NOTIFY_EXCHANGE']
        ))

        queue_name = self.config['RABBIT_QUEUE']
        result = channel.queue_declare(
            queue=queue_name,
            durable=True,
        )
        LOGGER.info('Declared queue {}'.format(
            queue_name
        ))

        for medium in self.config['MEDIUMS']:
            channel.queue_bind(
                exchange=self.config['RABBIT_NOTIFY_EXCHANGE'],
                queue=queue_name,
                routing_key=medium,
            )
            LOGGER.info('Bound queue with routing key: {}'.format(
                medium,
            ))

        channel.basic_consume(
            self.handler,
            queue=queue_name,
            consumer_tag=self.config['RABBIT_CONSUMER_TAG'],
            no_ack=True
        )

        try:
            channel.start_consuming()
        except KeyboardInterrupt:
            LOGGER.info('PVM stopped')



import logging

WORKERS = 2

# Rabbitmq
RABBIT_HOST = 'localhost'
RABBIT_NOTIFY_EXCHANGE = 'charpe_notify'
RABBIT_CONSUMER_TAG = 'charpe_notifier_consumer_1'
RABBIT_NO_ACK = True
RABBIT_QUEUE = 'charpe_queue'

# Log configuration
LOG_LEVEL = logging.INFO

# Time settings
TIMEZONE = 'UTC'

# Which available mediums we have
MEDIUMS = ['email', 'telegram']

# Needed to send mails
MAIL_SERVER = 'smtp.mailgun.org'
MAIL_PORT = '587'
MAIL_USE_TLS = True
MAIL_USE_SSL = False
MAIL_USERNAME = ''
MAIL_PASSWORD = ''
MAIL_DEFAULT_SENDER = 'charpe@example.com'
MAIL_SUPPRESS_SEND = False

# For template rendering
URL_PROTOCOL = 'https'
URL_SUBDOMAIN = 'com'

# For the telegram handler
TELEGRAM_BOT_KEY = '<bot key>'



#!/usr/bin/env python3
from setuptools import setup
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst')) as f:
    long_description = f.read()

setup(
    name='charpe',
    description='Sends messages from applications through different media',
    long_description=long_description,
    url='https://github.com/tracsa/charpe',

    version='1.0.0',

    author='Abraham Toriz Cruz',
    author_email='categulario@gmail.com',
    license='MIT',

    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='notification',

    packages=[
        'charpe',
    ],

    package_data={'charpe': ['templates/**']},

    entry_points={
        'console_scripts': [
            'charped = charpe.main:main',
        ],
    },

    include_package_data=True,

    install_requires=[
        'jinja2',
        'simplejson',
        'requests',
        'pika',
        'itacate',
        'pytz',
        'case_conversion',
    ],

    setup_requires=[
        'pytest-runner',
    ],

    tests_require=[
        'pytest',
        'pytest-mock',
    ],
)
